package eu.jonahbauer.wizard.core.machine.states.round;

import eu.jonahbauer.wizard.core.machine.Game;
import eu.jonahbauer.wizard.core.machine.GameData;
import eu.jonahbauer.wizard.core.messages.observer.HandMessage;
import eu.jonahbauer.wizard.core.model.Card;
import eu.jonahbauer.wizard.core.model.Deck;

import java.util.*;

public final class Dealing extends RoundState {
    private transient Map<UUID, List<Card>> hands;

    public Dealing(Game game, GameData data) {
        super(game, data);
    }

    @Override
    public void onEnter() {
        Deck deck = new Deck(getGame().getConfig().cards());
        deck.shuffle();

        hands = new HashMap<>();

        int dealer = getRound();
        int playerCount = getPlayerCount();
        int cardCount = getRound() + 1;
        for (int i = 1; i <= playerCount; i++) {
            int player = (dealer + i) % playerCount;
            hands.put(getPlayer(player), deck.draw(cardCount));
        }

        Optional<Card> trumpCard = Optional.ofNullable(deck.draw());

        transition(new DeterminingTrump(
                getGame(),
                getData().withHands(hands)
                         .withTrumpCard(trumpCard)
        ));
    }

    @Override
    public void onExit() {
        if (hands != null) {
            for (UUID player : getPlayers()) {
                getGame().notify(player, new HandMessage(player, hands.get(player)));
            }
        }
    }
}
