package eu.jonahbauer.wizard.core.machine.states.round;

import eu.jonahbauer.wizard.core.machine.Game;
import eu.jonahbauer.wizard.core.machine.GameData;
import eu.jonahbauer.wizard.core.messages.observer.TrumpMessage;
import eu.jonahbauer.wizard.core.messages.observer.UserInputMessage;
import eu.jonahbauer.wizard.core.messages.player.PickTrumpMessage;
import eu.jonahbauer.wizard.core.messages.player.PlayerMessage;
import eu.jonahbauer.wizard.core.model.Card;
import org.jetbrains.annotations.VisibleForTesting;

import java.util.Arrays;
import java.util.Collections;
import java.util.UUID;

import static eu.jonahbauer.wizard.core.messages.observer.UserInputMessage.Action.PICK_TRUMP;

/**
 *
 */
public final class DeterminingTrump extends RoundState {
    private transient Card.Suit trumpSuit;

    public DeterminingTrump(Game game, GameData data) {
        super(game, data.requireAllHands().requireTrumpCard());
    }

    @Override
    public void onEnter() {
        Card trumpCard = getTrumpCard();
        Card.Suit trumpSuit = trumpCard != null ? trumpCard.getTrumpSuit() : Card.Suit.NONE;
        if (trumpSuit == null) {
            getGame().notify(new TrumpMessage(getTrumpCard(), null));
            getGame().notify(new UserInputMessage(getDealer(), PICK_TRUMP, getTimeout(true)));
            timeout();
        } else {
            this.trumpSuit = trumpSuit;
            transition();
        }
    }

    @Override
    public void onTimeout() {
        Card.Suit[] suits = new Card.Suit[] {Card.Suit.BLUE, Card.Suit.GREEN, Card.Suit.RED, Card.Suit.YELLOW};
        this.trumpSuit = suits[(int)(Math.random() * suits.length)];
        transition();
    }

    @Override
    public void onExit() {
        if (trumpSuit != null) {
            getGame().notify(new TrumpMessage(getTrumpCard(), trumpSuit));
        }
    }

    @Override
    public void onMessage(UUID player, PlayerMessage message) {
        if (getDealer().equals(player) && message instanceof PickTrumpMessage trumpMessage) {
            checkTrumpSuit(trumpMessage.getTrumpSuit());
            this.trumpSuit = trumpMessage.getTrumpSuit();
            transition();
        } else {
            super.onMessage(player, message);
        }
    }

    @VisibleForTesting
    void checkTrumpSuit(Card.Suit suit) {
        Card.Suit[] suits = new Card.Suit[] {Card.Suit.BLUE, Card.Suit.GREEN, Card.Suit.RED, Card.Suit.YELLOW};
        for (Card.Suit s : suits) {
            if (s == suit) return;
        }
        throw new IllegalArgumentException("Trump suit must be one of " + Arrays.toString(suits) + ".");
    }

    private void transition() {
        if (trumpSuit == null) throw new AssertionError();

        transition(new Predicting(
                getGame(),
                getData().withTrumpSuit(trumpSuit)
                         .withPredictions(Collections.emptyMap())
                         .withCurrentPlayer(getDealer())
                         .withNextPlayer()
        ));
    }
}
