package eu.jonahbauer.wizard.core.machine.states.round;

import eu.jonahbauer.wizard.core.machine.Game;
import eu.jonahbauer.wizard.core.model.Configuration;
import org.junit.jupiter.api.Test;

import java.util.Map;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.Mockito.*;

public class PredictingTest {
    @Test
    public void checkPrediction_ThrowsIllegalArgument_IfOutOfBounds() {
        Map<UUID, Integer> predictions = Map.of();

        Configuration config = mock(Configuration.class);
        when(config.allowExactPredictions()).thenReturn(true);

        Game game = mock(Game.class);
        when(game.getConfig()).thenReturn(config);

        Predicting state = mock(Predicting.class);
        doCallRealMethod().when(state).checkPrediction(anyInt());
        when(state.getGame()).thenReturn(game);
        when(state.getPredictions()).thenReturn(predictions);
        when(state.getDealer()).thenReturn(new UUID(0,0));
        when(state.getCurrentPlayer()).thenReturn(new UUID(0,1));
        when(state.getRound()).thenReturn(10);

        assertThrows(IllegalArgumentException.class, () -> state.checkPrediction(-1));
        assertThrows(IllegalArgumentException.class, () -> state.checkPrediction(12));

        for (int i = 0; i < 12; i++) {
            state.checkPrediction(i);
        }
    }

    @Test
    public void checkPrediction_ThrowsIllegalArgument_IfAddsUp() {
        UUID player = new UUID(0,0);
        Map<UUID, Integer> predictions = Map.of(new UUID(0,1), 10);

        Configuration config = mock(Configuration.class);
        when(config.allowExactPredictions()).thenReturn(false);

        Game game = mock(Game.class);
        when(game.getConfig()).thenReturn(config);

        Predicting state = mock(Predicting.class);
        doCallRealMethod().when(state).checkPrediction(anyInt());
        when(state.getGame()).thenReturn(game);
        when(state.getPredictions()).thenReturn(predictions);
        when(state.getDealer()).thenReturn(player);
        when(state.getCurrentPlayer()).thenReturn(player);
        when(state.getRound()).thenReturn(10);

        assertThrows(IllegalArgumentException.class, () -> state.checkPrediction(1));

        for (int i = 0; i < 12; i++) {
            if (i == 1) continue;
            state.checkPrediction(i);
        }
    }
}
