package eu.jonahbauer.wizard.core.machine.states.trick;

import eu.jonahbauer.wizard.core.machine.states.trick.PlayingCard;
import eu.jonahbauer.wizard.core.messages.player.PickTrumpMessage;
import eu.jonahbauer.wizard.core.messages.player.PlayCardMessage;
import eu.jonahbauer.wizard.core.messages.player.PredictMessage;
import eu.jonahbauer.wizard.core.model.Card;
import org.junit.jupiter.api.Test;

import java.util.List;
import java.util.Map;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.Mockito.*;

public class PlayingCardTest {
    @Test
    public void checkCard_ThrowsIllegalArgument_IfCardNotInHand() {
        List<Card> hand = List.of(Card.BLUE_1, Card.RED_1, Card.YELLOW_1, Card.GREEN_JESTER, Card.BLUE_WIZARD);
        UUID player = new UUID(0, 0);

        PlayingCard state = mock(PlayingCard.class);
        doCallRealMethod().when(state).checkCard(any());
        when(state.getHands()).thenReturn(Map.of(player, hand));
        when(state.getStack()).thenReturn(List.of());
        when(state.getCurrentPlayer()).thenReturn(player);

        for (Card value : Card.values()) {
            if (hand.contains(value)) continue;
            assertThrows(IllegalArgumentException.class, () -> state.checkCard(value));
        }

        verify(state, times(0)).getData();
    }

    @Test
    public void checkCard_ThrowsIllegalArgument_IfNotFollowingSuit() {
        List<Card> hand = List.of(Card.BLUE_1, Card.RED_1, Card.YELLOW_1, Card.GREEN_JESTER, Card.BLUE_WIZARD);
        UUID player = new UUID(0, 0);

        PlayingCard state = mock(PlayingCard.class);
        doCallRealMethod().when(state).checkCard(any());
        when(state.getTrickSuit()).thenReturn(Card.Suit.YELLOW);
        when(state.getHands()).thenReturn(Map.of(player, hand));
        when(state.getCurrentPlayer()).thenReturn(player);

        assertThrows(IllegalArgumentException.class, () -> state.checkCard(Card.BLUE_1));
        assertThrows(IllegalArgumentException.class, () -> state.checkCard(Card.RED_1));

        state.checkCard(Card.YELLOW_1);
        state.checkCard(Card.GREEN_JESTER);
        state.checkCard(Card.BLUE_WIZARD);
    }

    @Test
    public void onMessage_ThrowsIllegalState_IfNotCurrentPlayer() {
        UUID player = new UUID(0, 0);
        UUID player2 = new UUID(0, 1);

        PlayingCard state = mock(PlayingCard.class);
        doCallRealMethod().when(state).onMessage(any(), any());
        when(state.getCurrentPlayer()).thenReturn(player);

        assertThrows(IllegalStateException.class, () -> state.onMessage(player2, new PlayCardMessage(Card.BLUE_WIZARD)));
        assertThrows(IllegalStateException.class, () -> state.onMessage(player2, new PredictMessage(1)));
        assertThrows(IllegalStateException.class, () -> state.onMessage(player2, new PickTrumpMessage(Card.Suit.BLUE)));
    }

    @Test
    public void onMessage_ThrowsIllegalState_IfNotPlayCard() {
        UUID player = new UUID(0, 0);

        PlayingCard state = mock(PlayingCard.class);
        doCallRealMethod().when(state).onMessage(any(), any());
        when(state.getCurrentPlayer()).thenReturn(player);

        assertThrows(IllegalStateException.class, () -> state.onMessage(player, new PredictMessage(1)));
        assertThrows(IllegalStateException.class, () -> state.onMessage(player, new PickTrumpMessage(Card.Suit.BLUE)));
    }
}
